var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, state, value, kind, f) {
    if (kind === "m") throw new TypeError("Private method is not writable");
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a setter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot write private member to an object whose class did not declare it");
    return (kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value)), value;
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
var _Pageres_options, _Pageres_stats, _Pageres_items, _Pageres_sizes, _Pageres_urls, _Pageres__source, _Pageres__destination;
import { parse as parseUrl } from 'node:url';
import path from 'node:path';
import fs from 'node:fs';
import fsPromises from 'node:fs/promises';
import os from 'node:os';
import { EventEmitter } from 'node:events';
import pMemoize from 'p-memoize';
import filenamify from 'filenamify';
import { unusedFilename } from 'unused-filename';
import arrayUniq from 'array-uniq';
import arrayDiffer from 'array-differ';
import dateFns from 'date-fns';
import getResolutions from 'get-res';
import logSymbols from 'log-symbols';
import makeDir from 'make-dir';
import captureWebsite from 'capture-website';
import viewportList from 'viewport-list';
import template from 'lodash.template';
import plur from 'plur';
import filenamifyUrl from 'filenamify-url';
import pMap from 'p-map';
const cpuCount = os.cpus().length;
const getResolutionsMemoized = pMemoize(getResolutions);
// @ts-expect-error - TS is not very smart.
const viewportListMemoized = pMemoize(viewportList);
// TODO: Use private class fields when targeting Node.js 12.
/**
Capture screenshots of websites in various resolutions. A good way to make sure your websites are responsive. It's speedy and generates 100 screenshots from 10 different websites in just over a minute. It can also be used to render SVG images.
*/
export default class Pageres extends EventEmitter {
    constructor(options = {}) {
        super();
        _Pageres_options.set(this, void 0);
        _Pageres_stats.set(this, {}); // eslint-disable-line @typescript-eslint/consistent-type-assertions
        _Pageres_items.set(this, []);
        _Pageres_sizes.set(this, []);
        _Pageres_urls.set(this, []);
        _Pageres__source.set(this, []);
        _Pageres__destination.set(this, '');
        // Prevent false-positive `MaxListenersExceededWarning` warnings
        this.setMaxListeners(Number.POSITIVE_INFINITY);
        __classPrivateFieldSet(this, _Pageres_options, { ...options }, "f");
        __classPrivateFieldGet(this, _Pageres_options, "f").filename = __classPrivateFieldGet(this, _Pageres_options, "f").filename ?? '<%= url %>-<%= size %><%= crop %>';
        __classPrivateFieldGet(this, _Pageres_options, "f").format = __classPrivateFieldGet(this, _Pageres_options, "f").format ?? 'png';
        __classPrivateFieldGet(this, _Pageres_options, "f").incrementalName = __classPrivateFieldGet(this, _Pageres_options, "f").incrementalName ?? false;
        __classPrivateFieldGet(this, _Pageres_options, "f").launchOptions = __classPrivateFieldGet(this, _Pageres_options, "f").launchOptions ?? {};
    }
    source(url, sizes, options) {
        if (url === undefined) {
            return __classPrivateFieldGet(this, _Pageres__source, "f");
        }
        if (!(typeof url === 'string' && url.length > 0)) {
            throw new TypeError('URL required');
        }
        if (!(Array.isArray(sizes) && sizes.length > 0)) {
            throw new TypeError('Sizes required');
        }
        __classPrivateFieldGet(this, _Pageres__source, "f").push({ url, sizes, options });
        return this;
    }
    destination(directory) {
        if (directory === undefined) {
            return __classPrivateFieldGet(this, _Pageres__destination, "f");
        }
        if (!(typeof directory === 'string' && directory.length > 0)) {
            throw new TypeError('Directory required');
        }
        __classPrivateFieldSet(this, _Pageres__destination, directory, "f");
        return this;
    }
    /**
    Run pageres.

    @returns List of screenshot buffer data.

    @example
    ```
    import Pageres from 'pageres';

    await new Pageres({delay: 2})
        .source('https://sindresorhus.com', ['1280x1024'])
        .destination('screenshots')
        .run();
    ```
    */
    async run() {
        await Promise.all(this.source().map(async (source) => {
            const options = {
                ...__classPrivateFieldGet(this, _Pageres_options, "f"),
                ...source.options,
            };
            const sizes = arrayUniq(source.sizes.filter(size => /^\d{2,4}x\d{2,4}$/i.test(size)));
            const keywords = arrayDiffer(source.sizes, sizes);
            __classPrivateFieldGet(this, _Pageres_urls, "f").push(source.url);
            if (sizes.length === 0 && keywords.includes('w3counter')) {
                return this.resolution(source.url, options);
            }
            if (keywords.length > 0) {
                return this.viewport({ url: source.url, sizes, keywords }, options);
            }
            const screenshots = await pMap(sizes, async (size) => {
                __classPrivateFieldGet(this, _Pageres_sizes, "f").push(size);
                return this.create(source.url, size, options);
            }, { concurrency: cpuCount * 2 });
            __classPrivateFieldGet(this, _Pageres_items, "f").push(...screenshots);
            return undefined;
        }));
        __classPrivateFieldGet(this, _Pageres_stats, "f").urls = arrayUniq(__classPrivateFieldGet(this, _Pageres_urls, "f")).length;
        __classPrivateFieldGet(this, _Pageres_stats, "f").sizes = arrayUniq(__classPrivateFieldGet(this, _Pageres_sizes, "f")).length;
        __classPrivateFieldGet(this, _Pageres_stats, "f").screenshots = __classPrivateFieldGet(this, _Pageres_items, "f").length;
        if (!this.destination()) {
            return __classPrivateFieldGet(this, _Pageres_items, "f");
        }
        await this.save(__classPrivateFieldGet(this, _Pageres_items, "f"));
        return __classPrivateFieldGet(this, _Pageres_items, "f");
    }
    /**
    Print a success message to the console.

    @example
    ```
    import Pageres from 'pageres';

    const pageres = new Pageres({delay: 2})
        .source('https://sindresorhus.com', ['1280x1024', '1920x1080'])
        .destination('screenshots');

    await pageres.run();

    // prints: Generated 2 screenshots from 1 url and 2 sizes.
    pageres.successMessage();
    ```
    */
    successMessage() {
        const { screenshots, sizes, urls } = __classPrivateFieldGet(this, _Pageres_stats, "f");
        const words = {
            screenshots: plur('screenshot', screenshots),
            sizes: plur('size', sizes),
            urls: plur('url', urls),
        };
        console.log(`\n${logSymbols.success} Generated ${screenshots} ${words.screenshots} from ${urls} ${words.urls} and ${sizes} ${words.sizes}`);
    }
    async resolution(url, options) {
        for (const item of await getResolutionsMemoized()) {
            __classPrivateFieldGet(this, _Pageres_sizes, "f").push(item.item);
            __classPrivateFieldGet(this, _Pageres_items, "f").push(await this.create(url, item.item, options));
        }
    }
    async viewport(viewport, options) {
        for (const item of await viewportListMemoized(viewport.keywords)) {
            __classPrivateFieldGet(this, _Pageres_sizes, "f").push(item.size);
            viewport.sizes.push(item.size);
        }
        for (const size of arrayUniq(viewport.sizes)) {
            __classPrivateFieldGet(this, _Pageres_items, "f").push(await this.create(viewport.url, size, options));
        }
    }
    async save(screenshots) {
        await Promise.all(screenshots.map(async (screenshot) => {
            await makeDir(this.destination());
            const dest = path.join(this.destination(), screenshot.filename);
            await fsPromises.writeFile(dest, screenshot);
        }));
    }
    async create(url, size, options) {
        const basename = fs.existsSync(url) ? path.basename(url) : url;
        let hash = parseUrl(url).hash ?? '';
        // Strip empty hash fragments: `#` `#/` `#!/`
        if (/^#!?\/?$/.test(hash)) {
            hash = '';
        }
        const [width, height] = size.split('x');
        const filenameTemplate = template(`${options.filename}.${options.format}`);
        const now = Date.now();
        let filename = filenameTemplate({
            crop: options.crop ? '-cropped' : '',
            date: dateFns.format(now, 'yyyy-MM-dd'),
            time: dateFns.format(now, 'HH-mm-ss'),
            size,
            width,
            height,
            url: `${filenamifyUrl(basename)}${filenamify(hash)}`,
        });
        if (options.incrementalName) {
            filename = await unusedFilename(filename);
        }
        // TODO: Type this using the `capture-website` types
        const finalOptions = {
            width: Number(width),
            height: Number(height),
            delay: options.delay,
            timeout: options.timeout,
            fullPage: !options.crop,
            styles: options.css && [options.css],
            defaultBackground: !options.transparent,
            scripts: options.script && [options.script],
            cookies: options.cookies,
            element: options.selector,
            hideElements: options.hide,
            scaleFactor: options.scale ?? 1,
            type: options.format === 'jpg' ? 'jpeg' : 'png',
            userAgent: options.userAgent,
            headers: options.headers,
            darkMode: options.darkMode,
            launchOptions: options.launchOptions,
            beforeScreenshot: options.beforeScreenshot,
        };
        if (options.username && options.password) {
            finalOptions.authentication = {
                username: options.username,
                password: options.password,
            };
        }
        const screenshot = await captureWebsite.buffer(url, finalOptions);
        screenshot.filename = filename;
        return screenshot;
    }
}
_Pageres_options = new WeakMap(), _Pageres_stats = new WeakMap(), _Pageres_items = new WeakMap(), _Pageres_sizes = new WeakMap(), _Pageres_urls = new WeakMap(), _Pageres__source = new WeakMap(), _Pageres__destination = new WeakMap();
